/*
 * MIT License
 *
 * Copyright (c) 2016 Dries007 & DoubleDoorDevelopment
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package net.doubledoordev.inventorylock.util;

import com.mojang.authlib.GameProfile;
import net.doubledoordev.inventorylock.InventoryLock;
import net.minecraft.block.Block;
import net.minecraft.block.BlockChest;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.server.management.PlayerProfileCache;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.text.Style;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.util.text.event.HoverEvent;
import net.minecraft.world.ILockableContainer;
import net.minecraft.world.LockCode;
import net.minecraftforge.common.util.FakePlayer;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.event.world.BlockEvent;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import java.util.UUID;

import static net.doubledoordev.inventorylock.util.Constants.*;
import static net.minecraftforge.common.util.Constants.NBT.TAG_STRING;

/**
 * @author Dries007
 */
public class ServerEventHandler
{
    public static final ServerEventHandler EVENT_HANDLER = new ServerEventHandler();

    private ServerEventHandler()
    {

    }

    @SubscribeEvent
    public void onBlockBreak(BlockEvent.BreakEvent event)
    {
        final int protection = InventoryLock.getBreakProtection();
        if (protection == 0) return;
        final EntityPlayer player = event.getPlayer();
        //noinspection ConstantConditions
        if (protection == 1 && (player == null || player instanceof FakePlayer || player.func_146103_bH() == null || player.func_110124_au() == null)) return;
        TileEntity te = event.getWorld().func_175625_s(event.getPos());
        if (!(te instanceof ILockableContainer)) return;
        if (!player.func_175146_a(((ILockableContainer) te).func_174891_i())) event.setCanceled(true);
    }

    @SubscribeEvent
    public void onPlayerInteractRightClickBlock(PlayerInteractEvent.RightClickBlock event)
    {
        EntityPlayer player = event.getEntityPlayer();
        if (player == null || player instanceof FakePlayer) return;
        ItemStack stack = event.getItemStack();
        if (stack == null) return;
        NBTTagCompound nbt = stack.func_179543_a(MOD_ID, false);
        //noinspection ConstantConditions
        if (nbt == null) return;

        event.setCanceled(true);
        Action action = Action.values()[nbt.func_74771_c(ACTION)];

        TileEntity te = event.getWorld().func_175625_s(event.getPos());
        if (te == null) return;
        if (!(te instanceof ILockableContainer))
        {
            player.func_146105_b(new TextComponentString("This block is not lockable :(").func_150255_a(new Style().func_150238_a(TextFormatting.RED)));
            return;
        }
        ILockableContainer lc = ((ILockableContainer) te);
        Block block = event.getWorld().func_180495_p(event.getPos()).func_177230_c();
        if (block instanceof BlockChest)
        {
            lc = ((BlockChest) block).func_180676_d(event.getWorld(), event.getPos());
            if (lc == null) lc = (ILockableContainer) te;
        }

        LockCode existingLc = lc.func_174891_i();

        if (action == Action.LOCK) // We want to lock
        {
            //noinspection ConstantConditions
            if (existingLc == null || existingLc.func_180160_a()) // There is no lock yet, OK
            {
                lc.func_174892_a(new BetterLockCode().add(player.func_110124_au()));
                player.func_146105_b(new TextComponentString("Locked!").func_150255_a(new Style().func_150238_a(TextFormatting.GREEN)));
            }
            else player.func_146105_b(new TextComponentString("This block is already locked.").func_150255_a(new Style().func_150238_a(TextFormatting.RED)));

            return; // End of any LOCK case
        }
        // Beyond here the block needs to already be locked, via OUR lock, and the player needs to have access.
        //noinspection ConstantConditions
        if (existingLc == null || existingLc.func_180160_a())
        {
            player.func_146105_b(new TextComponentString("This block is not locked.").func_150255_a(new Style().func_150238_a(TextFormatting.RED)));
            return;
        }
        if (!(existingLc instanceof BetterLockCode))
        {
            player.func_146105_b(new TextComponentString("This block is not locked via " + MOD_ID + ".").func_150255_a(new Style().func_150238_a(TextFormatting.RED)));
            return;
        }
        BetterLockCode blc = ((BetterLockCode) existingLc);
        if (!blc.contains(player))
        {
            player.func_146105_b(new TextComponentString("You do not have access to this block.").func_150255_a(new Style().func_150238_a(TextFormatting.RED)));
            return;
        }
        // LOCK is already handled.
        if (action == Action.UNLOCK) // We want to unlock (set back to the EMPTY_CODE singleton)
        {
            lc.func_174892_a(LockCode.field_180162_a);
            player.func_146105_b(new TextComponentString("Unlocked!").func_150255_a(new Style().func_150238_a(TextFormatting.GREEN)));
            return;
        }
        else if (action == Action.ADD) // We want to add uuids
        {
            NBTTagList list = nbt.func_150295_c(UUIDS, TAG_STRING);
            for (int i = 0; i < list.func_74745_c(); i++) blc.add(UUID.fromString(list.func_150307_f(i)));
        }
        else if (action == Action.REMOVE) // We want to remove uuids
        {
            NBTTagList list = nbt.func_150295_c(UUIDS, TAG_STRING);
            for (int i = 0; i < list.func_74745_c(); i++)
            {
                UUID uuid = UUID.fromString(list.func_150307_f(i));
                if (!player.func_110124_au().equals(uuid)) blc.remove(uuid); // Anti self lockout
            }
        }
        // Print this for ADD, REMOVE & INSPECT (LOCK & UNLOCK return early)
        player.func_146105_b(new TextComponentString("People with access:").func_150255_a(new Style().func_150238_a(TextFormatting.AQUA)));
        for (UUID uuid : blc.list)
        {
            PlayerProfileCache ppc = FMLCommonHandler.instance().getMinecraftServerInstance().func_152358_ax();
            GameProfile gp = ppc.func_152652_a(uuid);
            if (gp == null) player.func_146105_b(new TextComponentString(uuid.toString()).func_150255_a(new Style().func_150209_a(new HoverEvent(HoverEvent.Action.SHOW_TEXT, new TextComponentString("Missing username...")))));
            else player.func_146105_b(new TextComponentString(gp.getName()).func_150255_a(new Style().func_150209_a(new HoverEvent(HoverEvent.Action.SHOW_TEXT, new TextComponentString(uuid.toString())))));
        }
    }
}
